# testhand.py
# Steve Marschner (srm2) and Lillian Lee (ljl2)
# March 19, 2013
"""Testing module for the hand module."""

import hand

    
# Each entry in the following list is a list of a hand and the correct
# classifications for that hand.
# For each hand, the answers are in the order: [is_pair, is_2pair, is_3kind,
# is_straight, is_flush, is_full_house, is_4kind, is_straight_flush]
test_data_5cards = [  # P,2P,3K, S, F,FH,4K,SF
    ['2C 2D 3H 5C 6D', [1, 0, 0, 0, 0, 0, 0, 0]], # just 1 pair
    ['2C 2D 2H 5C 6D', [1, 0, 1, 0, 0, 0, 0, 0]], # 3 of a kind, also 1 pair
    ['2C 2D 5H 5C 6D', [1, 1, 0, 0, 0, 0, 0, 0]], # 2 pair, also 1 pair
    ['2C 2D 2H 3C 3D', [1, 1, 1, 0, 0, 1, 0, 0]], # full house, also 2p, 3k, and 1p
    ['2C 3D 4H 5C 6D', [0, 0, 0, 1, 0, 0, 0, 0]], # low straight
    ['6D 7D 8D 9D TD', [0, 0, 0, 1, 1, 0, 0, 1]], # 10-high straight flush
    ['9C 4D 4H 4C 4S', [1, 0, 1, 0, 0, 0, 1, 0]], # 4 of a kind, also 3k and 1p
    ['AS KS 4S 7S 5S', [0, 0, 0, 0, 1, 0, 0, 0]], # flush
    ['KS JH QC AC TH', [0, 0, 0, 1, 0, 0, 0, 0]], # high straight
    ['KS JS QS AS TS', [0, 0, 0, 1, 1, 0, 0, 1]], # straight flush is str and fl
]

test_data_7cards = [        # P 2 3 S F H 4 SF
    ['2C 2D 3H KS TH 5C 6D', [1,0,0,0,0,0,0,0]], # just 1 pair
    ['AS KH 2C 2D 2H 5C 6D', [1,0,1,0,0,0,0,0]], # 3 of a kind, also 1 pair
    ['3H 2C 2D 6S 5H 5C 6D', [1,1,0,0,0,0,0,0]], # 2 pair, also 1 pair
    ['2C 4C 2D 2H 3C 6H 3D', [1,1,1,0,0,1,0,0]], # full house, also 2p, 3k, and 1p.
    ['AS 2C 3D 4H 5C 6D 7D', [0,0,0,1,0,0,0,0]], # low straight
    ['4H 5C 6D 7D 8D 9D TD', [0,0,0,1,1,0,0,1]], # 10-high straight flush
    ['9C 4D 4H 3C 5C 4C 4S', [1,0,1,0,0,0,1,0]], # 4 of a kind, also 3k and 1p
    ['9C 4D 4H 5S 5C 4C 4S', [1,1,1,0,0,1,1,0]], # 4 of a kind, also f_h, 3k, 2p and 1p
    ['AS KS 4S 6H KS KS KS', [1,0,1,0,1,0,1,0]], # 4 of a kind, also a flush
    ['KS JH QC 2S 6H AC TH', [0,0,0,1,0,0,0,0]], # high straight
    ['4H 8S KS JS QS AS TS', [0,0,0,1,1,0,0,1]], # straight flush is str and fl
    ['4H 8S KS JH QS AS TS', [0,0,0,1,1,0,0,0]], # str and fl but not straight flush
    ['4H 8H AS 6H TC 5H 7H', [0,0,0,1,1,0,0,1]], # straight flush is str and fl
    ['4H 8H 6S 5S 7S AS TS', [0,0,0,1,1,0,0,0]], # str and fl but not straight flush
]

def test_hand_types(testcases):
    """A test to be sure all hand types can be correctly classified."""
    for testcase in testcases:
        h = hand.Hand(code=testcase[0]) # the hand
        answers = testcase[1] # the correct answers
        assert h.is_pair() == answers[0], repr(h) + ".is_pair() should be " + str(answers[0])
        assert h.is_2pair() == answers[1], repr(h) + ".is_2pair() should be " + str(answers[1])
        assert h.is_3kind() == answers[2], repr(h) + ".is_3kind() should be " + str(answers[2])
        assert h.is_straight() == answers[3], repr(h) + ".is_straight() should be " + str(answers[3])
        assert h.is_flush() == answers[4], repr(h) + ".is_flush() should be " + str(answers[4])
        assert h.is_full_house() == answers[5], repr(h) + ".is_full_house() should be " + str(answers[5])
        assert h.is_4kind() == answers[6], repr(h) + ".is_4kind() should be " + str(answers[6])
        assert h.is_straight_flush() == answers[7], repr(h) + ".is_straight_flush() should be " + str(answers[7])

if __name__ == '__main__':
    test_hand_types(test_data_5cards)
    print "passed 5-card tests"
    test_hand_types(test_data_7cards)
    print "passed 7-card tests"
    print "Hey, it works!"